# Progressive Compression of Triangle Meshes

* IPOL link:
* Version number: no release for MEPP2 and no recent tag; 
* last MEPP2 commit: 1a3327b099af5133b82f30301689743b07d3bb27 
* Last modification date: 2022/09/23

## Authors
* Vincent Vidal vincent.vidal@liris.cnrs.fr
* Lucas Dubouchet lucas@dubouchet.eu
* Guillaume Lavoué guillaume.lavoue@liris.cnrs.fr
* Pierre Alliez pierre.alliez@inria.fr

## Future releases and updates:
https://github.com/MEPP-team/MEPP2/

## Description
Implementation of Compressed Progressive Meshes [R. Pajarola and J. Rossignac, 
IEEE Transactions on Visualization and Computer Graphics, 6 (2000), pp.79-93].

We provide 2 main filters in the src/FEVV/Filters/CGAL/Progressive_Compression folder:
* progressive_compression_filter.hpp to compress a triangle mesh using our CPM implementation => Algorithm 1
* progressive_decompression_filter.hpp to decompress a binary compressed mesh file => Algorithm 2

The 2 filters can process the following halfedge-like data structure implementation:
OpenMesh, CGAL Surface Mesh, CGAL Polyhedral Surface, and CGAL Linear Cell Complex. 
Note that only MEPP2 release works with OpenMesh for the decompression. 
OpenMesh is not available in the IPOL release. 
OpenMesh does not work with the compression in the MEPP2 release, because the distance computation 
(to generate RD data) use a CGAL package that is not yet compatible with OpenMesh (see the paper for more details). 

### src/Examples/CGAL/Surface_mesh
This folder contains the code of 2 applications (with a main function).
See Demo description below.

### src/FEVV
This folder contains the API header only source code.

### src/Testing
This folder is dedicated to run tests.

#### src/Testing/CGAL
This folder contains the unit tests to run on CGAL data structures (LCC, Polyhedron, and Surface_mesh).

### src/Testing/Draco
This folder contains a Draco unit test to make sure Draco based code compiles and links properly.

### src/Testing/Utils
This folder contains some utils function, e.g., to compare two .off meshes.  

### src/Testing/Data
This folder contains the test data (e.g., mesh files). 

## Third party lib

Boost, CGAL, Eigen 3, google draco

### ext
This folder contains external library archives to be independent of 
distribution packages. It also has an install script (installation_script)
to compile and install these external libraries (it also compiles
the project at the end).

# Build / Compilation instructions
This project build relies on CMake (https://cmake.org/).
You can use MEPP2 and follow the MEPP2 installation guide (Windows, Linux, and Mac OS X): https://github.com/MEPP-team/MEPP2/blob/master/Install.md

You can also follow the instructions below if you use the IPOL link (Linux only).

## Ubuntu 22.04 LTS installation (or Ubuntu 22.04 docker on Windows) for the IPOL link

### Install dependencies

Scripting commands for installing dependencies (if you are admin you do not need sudo):
````
  # often needed...
  $ sudo apt update

  # C++ compilers
  $ sudo apt install -y build-essential
  
  # CMake
  $ sudo apt install -y cmake

  # Boost
  $ sudo apt install -y libboost-all-dev

  # Eigen 3
  $ sudo apt install -y libeigen3-dev

  # CGAL
  $ sudo apt install -y libgmp-dev libmpfr-dev
  $ sudo apt install -y libcgal-dev
  
  # google draco (>1.3.6)
  $ sudo apt install -y libdraco-dev 
````

### Build stage

Scripting commands for compiling the source code:
````
  # get source_code
  $ wget urlcode + decompress archive if needed

  # create build directory (here source_code=ipol_code_folder/src)
  $ cd source_code && mkdir build && cd build

  # compile Release
  with draco:
  $ cmake -DBUILD_USE_CGAL=ON -DBUILD_USE_DRACO=ON -DCMAKE_BUILD_TYPE=Release ..
  without draco:
  $ cmake -DBUILD_USE_CGAL=ON -DBUILD_USE_DRACO=OFF -DCMAKE_BUILD_TYPE=Release ..
  
  $ make -j 3
  
  # compile Debug 
  $ cmake -DBUILD_USE_CGAL=ON -DBUILD_USE_DRACO=ON -DCMAKE_BUILD_TYPE=Debug ..
  
  $ make  
````

## Linux installation independent of distribution packages

### Install dependencies and build projet in release mode

Scripting commands for installing dependencies (if you are admin you do not need sudo):
````
  # often needed...
  $ sudo apt update

  # C++ compilers
  $ sudo apt install -y build-essential
  
  # CMake
  $ sudo apt install -y cmake

  # get source_code
  $ wget urlcode + decompress archive if needed
  
  # go to ext folder
  $ cd ipol_code_folder/ext
  
  # run installation script (build and install 
  # Boost, CGAL, Eigen 3, and google draco).
  # It also creates ipol_code_folder/src/build
  # directory and compiles the project in release
  # mode with draco enabled. It then runs the 
  # ctest command. 
  $ sudo chmod +x installation_script
  $ ./installation_script
````

### Build stage

It works if the ./installation_script has been run
(the build folder has been created). 
You should be in the ipol_code_folder/ext directory.
````
  # in the ext folder
  $ MAINDIR=$(pwd)/..
  
  # go to build folder
  $ cd ${MAINDIR}/src/build

  # compile Release
  with draco:
  $ cmake -DBUILD_USE_CGAL=ON -DBUILD_USE_DRACO=ON -DCMAKE_BUILD_TYPE=Release -DEIGEN3_INCLUDE_DIR=${MAINDIR}/ext/eigen-3.4.0/ -DCGAL_DIR=${MAINDIR}/ext/CGAL-5.4/ -DGMP_LIBRARIES=${MAINDIR}/ext/gmp-4.2.3/distrib/lib/libgmp.so -DGMP_INCLUDE_DIR=${MAINDIR}/ext/gmp-4.2.3/distrib/include -DMPFR_LIBRARIES=${MAINDIR}/ext/mpfr-3.0.0/distrib/lib/libmpfr.so -DMPFR_INCLUDE_DIR=${MAINDIR}/ext/mpfr-3.0.0/distrib/include -DBOOST_ROOT=${MAINDIR}/ext/boost_1_74_0 -DDRACO_DIR=${MAINDIR}/ext/draco-1.5.2/distrib ..
  without draco:
  $ cmake -DBUILD_USE_CGAL=ON -DBUILD_USE_DRACO=OFF -DCMAKE_BUILD_TYPE=Release -DEIGEN3_INCLUDE_DIR=${MAINDIR}/ext/eigen-3.4.0/ -DCGAL_DIR=${MAINDIR}/ext/CGAL-5.4/ -DGMP_LIBRARIES=${MAINDIR}/ext/gmp-4.2.3/distrib/lib/libgmp.so -DGMP_INCLUDE_DIR=${MAINDIR}/ext/gmp-4.2.3/distrib/include -DMPFR_LIBRARIES=${MAINDIR}/ext/mpfr-3.0.0/distrib/lib/libmpfr.so -DMPFR_INCLUDE_DIR=${MAINDIR}/ext/mpfr-3.0.0/distrib/include -DBOOST_ROOT=${MAINDIR}/ext/boost_1_74_0 -DDRACO_DIR=${MAINDIR}/ext/draco-1.5.2/distrib ..
  
  $ make -j 3
  
  # compile Debug 
  $ cmake -DBUILD_USE_CGAL=ON -DBUILD_USE_DRACO=ON -DCMAKE_BUILD_TYPE=Debug -DEIGEN3_INCLUDE_DIR=${MAINDIR}/ext/eigen-3.4.0/ -DCGAL_DIR=${MAINDIR}/ext/CGAL-5.4/ -DGMP_LIBRARIES=${MAINDIR}/ext/gmp-4.2.3/distrib/lib/libgmp.so -DGMP_INCLUDE_DIR=${MAINDIR}/ext/gmp-4.2.3/distrib/include -DMPFR_LIBRARIES=${MAINDIR}/ext/mpfr-3.0.0/distrib/lib/libmpfr.so -DMPFR_INCLUDE_DIR=${MAINDIR}/ext/mpfr-3.0.0/distrib/include -DBOOST_ROOT=${MAINDIR}/ext/boost_1_74_0 -DDRACO_DIR=${MAINDIR}/ext/draco-1.5.2/distrib ..
  
  $ make  
````

## Demos and tests 

Demos (i.e., the CLI applications), the compare_mesh_files tool, and unit testing binaries 
are located within the build/bin subfolder. 
The unit test executables have their name starting with the "test_" prefix. 

### Demos

Compression demo usage ([] means optional):
```
./prog_compression_sm input_off_or_obj_mesh_file_name [ mode [output_binary_file_name [output_folder] ]
```
- mode is an integer: 0 for a single compression, 1 for several compressions (using available predictions, position types, and metrics), 2 for
                      distortions mode (between the input mesh with unquantized coordinates and several meshes (the LoDs with quantized coordinates) 
					  that are located within the folder name (the fourth argument in this case))
```
./prog_compression_sm input_off_or_obj_mesh_file_name 0 "" "" prediction_type metric_type position_type nb_max_batches nb_min_nb_v use_mean_th nb_q_bits 
```		
- prediction_type is an integer: 0 for no-prediction, 1 for delta prediction, and 2 for butterfly prediction
- metric_type is an integer: 0 for QEM, 1 for edge length, and 2 for local absolute volume error
- position_type is an integer: 0 for midpoint kept position and 1 for edge target kept position
- nb_max_batches is an integer which set the maximal number of decimation batches (to monitor the maximal number of different LoDs)
- nb_min_nb_v is an integer which sets the number of vertices below which the batch decimation stops; use -1 to set it to 5% of the input mesh vertices			
- use_mean_th is an integer: 0 for collapsing all collapsible edges and 1 for collapsing only edges whose cost is below the mean cost of the current decimation batch  
- nb_q_bits is an integer which sets the number of bits for the uniform quantization of XYZ coordinates: 10, 12 or 16 are common values

Decompression demo usage ([] means optional):
```
./prog_decompression_sm input_binary_file_name [output_off_or_obj_mesh_file_name [nb_max_batches]]
```
- input_binary_file_name is a compressed mesh file using the progressive compression executable. 
- nb_max_batches is an integer which sets the maximal number of batches to reconstruct (e.g., to
  reconstruct less batches than the total number of available batches).

### Run tests

````
$ ctest
````

## Experiments

The `data_experiments` folder contains some of the files used to generate the experimental data for ipol paper.
Note that you need to decompress this folder.

To get the running times, you need to uncomment the line "//#define TIMING"
in progressive_compression_filter.hpp and progressive_decompression_filter.hpp files
(in `src/FEVV/Filters/CGAL/Progressive_Compression` folder).
You then need to compile and run on the models (using the 0 mode, the -1 nb_min_nb_v, and 
the 70 nb_max_batches for the progressive compression).  
```
./prog_compression_sm input_off_or_obj_mesh_file_name 0 "" "" prediction_type metric_type position_type 70 -1 0 nb_q_bits 
```	
```
./prog_decompression_sm input_binary_file_name
```

To get the bpv and distortion cvs data, you need to activate the measure mode at line 196 of 
progressive_compression_filter.hpp file. Distortion measures are taken every five batches.
Reverse information measures are not zero only for the halfedge/target position type (no 
reverse information for the mid-point position type). 
 - When you run the compression demo in the 0 mode (single compression), then you generate 
   only one cvs data file.
 - When you run the compression demo in the 1 mode (measure mode), then you generate as many 
   cvs data files as there are predictors X position_types X local_metrics X quantizations
   specified at lines 267 to 282 of initializer_compression.h file (in `src/FEVV/Filters/CGAL/Progressive_Compression/Helpers` folder).

# Licensing / Copyright: See LICENSE-lgpl-3.0.txt

## Known issues
Non-manifold mesh (e.g. cut vertices); Non pure triangular meshes. 
The source code has been intensely tested on closed triangular 2-manifold meshes and tested on 2-manifold meshes with borders.