##### CGAL package finding
# Caveat emptor: for some undocumented reason CGAL package detection messes
#     up (it clears it completely at least on OSX) the content of the
#     Boost_LIBRARIES variable as set up by the Boost package detection.
#     In order to avoid this side effect, CGAL package detection must be
#     placed BEFORE Boost package detection.
if( BUILD_USE_CGAL )
  add_definitions( -DCGAL_EIGEN3_ENABLED ) ##### SOME CGAL FUNCTIONALITIES DEPEND ON EIGEN3

  set(CGAL_DONT_OVERRIDE_CMAKE_FLAGS TRUE CACHE BOOL "Prevent CGAL to override CMAKE flags")
  # The find_package( CGAL 4.7 REQUIRED ) directive fails on:
  #  * OSX 10.11 with a homebrew install
  #  * Ubuntu (as offered by Travis).
  # This might be due to CGAL config file that doesn't seem to positionate
  # version numbers properly. Indeed cmake 3.4 expects the config file to define
  # variable like "<package>_VERSION_MAJOR" whereas (at least on OSX 10.11 with
  # Homebrew packages) CGALConfig.cmake defines CGAL_MAJOR_VERSION
  # Refer to https://cmake.org/cmake/help/v3.4/command/find_package.html
  # Hence the following manual check
  find_package( CGAL REQUIRED CONFIG )
  if( ${CGAL_MAJOR_VERSION}.${CGAL_MINOR_VERSION} VERSION_LESS "4.14" )
    message (FATAL_ERROR "CGAL required minimum version is 4.14 - Found: ${CGAL_MAJOR_VERSION}.${CGAL_MINOR_VERSION}" )
    return()
  endif()
  add_definitions( -DFEVV_USE_CGAL )
  # FIXME: used in if and else (see below). Factorize this !
  add_definitions( -DBOOST_ALL_DYN_LINK )

  message( STATUS "Use CGAL version ${CGAL_MAJOR_VERSION}.${CGAL_MINOR_VERSION}.${CGAL_BUGFIX_VERSION}" )
else()
  add_definitions( -DBOOST_ALL_DYN_LINK )
  # Refer to Doc/Devel/CMakeFiles.md entry 003
  add_definitions( -DCGAL_NO_AUTOLINK_CGAL )
  add_definitions( -DCGAL_NDEBUG )
  add_definitions( -DCGAL_HEADER_ONLY )
  add_definitions( -DCGAL_DISABLE_GMP )
  message ( STATUS "Use local LGPL CGAL")
endif()

##### Boost package finding (mandatory)
find_package(Boost COMPONENTS thread system filesystem chrono REQUIRED)
if(Boost_FOUND)
  include_directories(${Boost_INCLUDE_DIRS})
  add_definitions( -DBOOST_BIND_GLOBAL_PLACEHOLDERS ) # for Boost >= 1.73, silence a warning about a deprecated use of boost bind
else()
  message(FATAL_ERROR "Boost not found. Please install it.")
endif(Boost_FOUND)

##### EIGEN3 package finding (mandatory)
if (WIN32)
	if(DEFINED ENV{EIGEN3_INCLUDE_DIR})
		SET( EIGEN3_INCLUDE_DIR $ENV{EIGEN3_INCLUDE_DIR} )
	ENDIF()
endif (WIN32)
# Eigen is needed except for the documentation
if( BUILD_USE_CGAL OR BUILD_USE_OPENMESH OR BUILD_USE_AIF )
	FIND_PACKAGE(Eigen3)
	if(EIGEN3_FOUND)
	  INCLUDE_DIRECTORIES(${EIGEN3_INCLUDE_DIR})
	ELSE(EIGEN3_FOUND)
	  MESSAGE(FATAL_ERROR
			  "Eigen3 not found. Please set EIGEN3_INCLUDE_DIR to the root directory of your Eigen3 installation.")
	endif(EIGEN3_FOUND)
endif()


##### Boost.Beast and OpenSSL - Early support ! Need Boost >= 1.70
if( BUILD_USE_BOOST_BEAST AND BUILD_USE_OPENSSL )
  find_package(OpenSSL REQUIRED)
  if ( OpenSSL_FOUND )
    include_directories( ${OPENSSL_INCLUDE_DIR} )
    add_definitions( -DFEVV_USE_OPENSSL )

    message( "---> OPENSSL_INCLUDE_DIR: ${OPENSSL_INCLUDE_DIR}" )
    message( "---> OPENSSL_LIBRARIES: ${OPENSSL_LIBRARIES}" )
  else()
    message( FATAL_ERROR
             "OpenSSL not found. Please set OPENSSL_ROOT_DIR or turn BUILD_USE_OPENSSL to OFF.")
  endif ()
endif ()

##### Draco package finding
if( BUILD_USE_DRACO )
  find_package(Draco)
  if ( draco_FOUND )
    include_directories( ${draco_INCLUDE_DIRS} )
    add_definitions( -DFEVV_USE_DRACO )
  else()
    message( FATAL_ERROR
             "Draco not found. Please set DRACO_DIR or turn BUILD_USE_DRACO to OFF.")
  endif ()
endif ()

